/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <readline/readline.h>
#include <readline/history.h>

#include "declaracao.h"
#include "cmd.h"

static int readline_support = 1;

static int fileprocess(struct declaracao *dec, int fd)
{
	char *line = NULL;
	size_t lsz = 0;
	FILE *f;
	int r;
	int rc = 0;
	int n = 1;
	f = fdopen(fd, "r");
	if (!f)
		return -errno;
	while (getline(&line, &lsz, f) > 0) {
		r = cmd_run(dec, line);
		if (r < 0) {
			fprintf(stderr, "Não foi possível executar comando na linha %d: %s\n",
				n, dec->error ?: strerror(-r));
			rc = r;
		}
		n++;
	}
	free(line);
	fclose(f);
	return rc;
}

static int ttyprocess(struct declaracao *dec)
{
	char *line = NULL;
	int r;
	using_history();
	while ((line = readline("declara> ")) != NULL) {
		if (*line)
			add_history(line);
		r = cmd_run(dec, line);
		if (r < 0) {
			fprintf(stderr, "Não foi possível executar comando: %s\n",
				dec->error ?: strerror(-r));
		}
		free(line);
	}
	return r;
}

static int process(char *filename)
{
	int r = 0;
	int fd;
	struct declaracao *dec;
	dec = declaracao_new(-1);
	if (!dec)
		return -errno;
	if (filename != NULL) {
		fd = open(filename, O_RDONLY);
		if (fd < 0) {
			r = -errno;
			goto out_open;
		}
		r = fileprocess(dec, fd);
		close(fd);
	} else {
		if (isatty(0) && readline_support) {
			r = ttyprocess(dec);
		} else {
			r = fileprocess(dec, 0);
		}
	}
out_open:
	declaracao_free(dec);
	return r;
}

static void usage(void)
{
	fprintf(stderr, "declara <filename>\n");
	exit(1);
}

int main(int argc, char **argv)
{
	char *filename = NULL;
	int r;
	if (!readline_support && argc < 2)
		usage();

	cmd_init();
	dec_cmd_init();

	if (argc > 1)
		filename = argv[1];
	r = process(filename);
	if (r)
		return 1;

	return 0;
}
